# HSPLITD - Split a single multispec hectospec file into 1-D spectra
# By Doug Mink, SAO Telescope Data Center
# March 5, 2008

procedure hsplitd (msfile)

string	msfile=""	{prompt="Aperture reference multispec file (drop .ms)"}
file	outdir=""	{prompt="Output directory for 1-D spectra (returned)"}
bool	deltemp=yes	{prompt="Delete temporary list file (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}
bool	debug=no	{prompt="Print more information (y or n)"}

begin

	file	keyfile, keypath, imin, imroot, imnew
	file	tempfile, tempfile2, outroot, temparg, newpath
	int	lroot, aperture, nfiles, i
	string	arg, tchar, ims, history, aparg
	string	keyword, apkwr, apval1, type, x1, x2, range
	string	name, ra, dec, catnum, fiber, x, y
	string	apkey, fpkey, objkey, skykey, fibkey
	string	pra, pdec, pepoch, apname
	bool	vb, db

	vb = verbose
	db = debug
	imin = msfile
#	keyfile = osfn (keywords)
	keyfile = ""

#  Set defaults for scopy program
	scopy.w1 = INDEF
	scopy.w2 = INDEF
	scopy.bands = "1"
	scopy.beams = ""
	scopy.apmodulus = 0
	scopy.format = "onedspec"
	scopy.renumber = no
	scopy.offset = 0
	scopy.clobber = yes
	scopy.merge = yes
	scopy.rebin = no
	scopy.verbose = yes

	if (imin == "") {
	    print "hsplit: Make 1D spectra (fiber.object) from Hectospec multispec file"
	    return
	    }

	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

# Make sure that input is a multispec file
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imroot = substr (imin, 1, lroot)
	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imnew = imin // ".fits"
		if (access (imnew)) {
		    imin = imnew
		    }
		else {
		    print ("HSPLITD: ", imin," is not a multispec file")
		    return
		    }
		}
	    }
        else {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		imnew = imroot // ".ms.fits"
		if (access (imnew)) {
		    imin = imnew
		    }
		else {
		    print ("HSPLITD: ", imin," is not a multispec file")
		    return
		    }
		}
	    else {
		imroot = imin
		imin = imroot // ".ms.fits"
		if (!access (imin)) {
		    print ("HSPLITD: ", imin," is not a multispec file")
		    return
		    }
		}
            }

# COMMENT OUT this section if HSPLITD is expected to always run
# Do not split an image twice
#	imgetp.default = "0"
#	imgetp (imin, "HSPLIT")
#	if (imgetp.value != "0") {
#	    if (vb) {
#		print ("HSPLITD: ", imin, " has already been split")
#		print ("HSPLIT = ", imgetp.value)
#		}
#	    return
#	    }


	if (vb) {
	    print ("Delete keywords listed in file ",imin)
	    }
	if (vb) {
	    print ("Object name and position from ", imin)
	    }

# Make separate file to pre-edit header for output files
	tempfile = imroot // "t.ms.fits"
	if (access (tempfile)) {
	    delete (tempfile)
	    }

# Copy .fits or .imh output from DOFIBERS into file to be edited
	imcopy (imin, tempfile)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " image split to " // "1d." // imroot
	hedit (tempfile, "HSPLIT", history, add+, verify-, show-)

# Get number of spectra in multispec file
	imgetp.default = "0"
	imgetp (imin, "i_naxis2")
	nfiles = int (imgetp.value)
	aparg = "1-" // nfiles

# If 1d directory does not exist, create it
	outdir = "1d." // imroot
	if (!access (outdir)) {
	    if (vb)
		print ("HSPLITD: Making directory ", outdir)
	    mkdir (outdir)
	    }
	outroot = outdir // "/temp"

# Extract all spectra into a separate FITS file
	if (vb) {
	    print ("scopy ", tempfile, " ", outroot, " apertures=", aparg)
	    }
	scopy (tempfile, outroot, apertures=aparg)

	for (i = 1; i <= nfiles; i = i + 1) {

	    if (i < 10) {
		apname = "000" // i
		}
	    else if (i < 100) {
		apname = "00" // i
		}
	    else if (i < 1000) {
		apname = "0" // i
		}
	    else {
		apname = i
		}
	    tempfile2 = outdir // "/temp." // apname // ".fits"

	    haphead (tempfile2, verbose=vb, debug=db)
	    name = haphead.newname

# Rename file to aperture number and object name
	    newpath = outdir // "/" // name
	    rename (tempfile2, newpath)
	    if (vb) {
		print ("rename ",tempfile2, " ", newpath)
		}
	    }

	if (deltemp) {
	    delete (tempfile)
	    }

# Add history line including time to original multispec file
	gettime ()
	history = gettime.date // " " // gettime.time // " image split to " // "1d." // imroot
	hedit (imin, "HSPLIT", history, add+, verify-, show=vb)
	return
end

# Mar  5 2008	New task based on HSPLIT with returned output directory param
