# HEXTRACT -- Extract  Hectospec image into multispec file
# By Doug Mink, SAO Telescope Data Center
# March 3, 2008

procedure hextract (images)

file	images=""	{prompt="Input image(s) to extract"}
file	mapfile=""	{prompt="Fiber to sky map file"}

file	mapdir=""	{prompt="Directory for mapfiles"}
bool	mapfix=no	{prompt="Replace tabs and fix names in map file"}
file	apref=""	{prompt="Aperture reference spectrum"}
int	fibers=300	{prompt="Number of fibers"}
real	width=8.	{prompt="Width of profiles (pixels)"}
real	minsep=5.	{prompt="Minimum separation between fibers (pixels)"}
real	maxsep=100. 	{prompt="Maximum separation between fibers (pixels)"}
real	minshift=0.01	{prompt="Minimum shift to use in extraction (pixels)"}
string	readnoise="1"	{prompt="Read out noise sigma (photons)"}
string	gain="1"	{prompt="Photon gain (photons/data number)"}
bool	resize=no	{prompt="Resize object apertures (y or n)"}
bool	recenter=no	{prompt="Recenter object apertures (y or n)"}
bool    shift=no	{prompt="Use average shift instead of recentering (y or n)"}
real    npeaks=0.2	{prompt="Select brightest peaks"}
bool	splot=no	{prompt="Plot extracted spectra using splot (y or n)"}
bool	deltemp=yes	{prompt="Delete temporary file (y or n)"}
bool	verbose=yes	{prompt="Log progress (y or n)"}
bool	debug=yes	{prompt="Print extra information for debugging (y or n)"}

struct *exlist

begin
	bool vb, db, dt, sp
	file imin, templist
	string ims

	ims = images
	vb = verbose
	db = debug
	dt = deltemp
	sp = splot

#  Make temporary list of input files
	templist = mktemp ("tempex") // ".list"
	files (ims, >templist)

# Process list of files one at a time
	exlist = templist
	while (fscan (exlist,imin) != EOF) {
	    hextract1.mapfile = mapfile
	    hextract1.mapdir = mapdir
	    hextract1.mapfix = mapfix
	    hextract1.apref = apref
	    hextract1.fibers = fibers
	    hextract1.width = width
	    hextract1.minsep = minsep
	    hextract1.maxsep = maxsep
	    hextract1.minshift = minshift
	    hextract1.readnoise = readnoise
	    hextract1.gain = gain
	    hextract1.resize = resize
	    hextract1.recenter = recenter
	    hextract1.shift = shift
	    hextract1.npeaks = npeaks
	    hextract1 (imin, splot=sp, deltemp=dt,verbose=vb,debug=db)
	    }

	if (deltemp)
	    delete (templist)

	exlist = ""
end

# Jun 15 2004	Add recenter, shift, npeaks, parameters for dohspec and proc
# Jun 16 2004	Add resize parameter for dohspec and apscript
# Sep  2 2004	Add HEXTRACT keyword when processed
# Oct 18 2004	Print HEXTRACT keyword if already processed

# Jan 12 2005	Set more dohspec parameters from hextract parameters
# Mar 21 2005	Image map name can be made from input file name .fits -> _map
# Mar 21 2005	Do not change deltemp if reading from file list
# Mar 21 2005	Read from list of .ms.fits output files or .fits input files
# Mar 24 2005	Change temporary file from "mergelist" to "extractlist"
# Mar 29 2005	WFW - turn off splot at end unless verbose or debug
# Apr  4 2005	DJM - combine changes made by Bill and Doug
# May 17 2005	DJM - Use HSHIFT shifts from header on aperture reference
# Jul  6 2005	DJM - Check for map files with exposure sequence in them

# May 10 2006	DJM - Add to hsmapfix comment
# Nov 14 2006	DJM - Add mapfix parameter

# Sep 11 2007	DJM - Continue even if no map file is found
# Sep 13 2007	DJM - Copy @list to templist so it can be deleted
# Oct 31 2007	DJM - Do not set dohspec.clean so it can be set independently

# Feb 25 2008	DJM - Move actual extraction to HEXTRACT1
# Mar  3 2008	DJM - Add splot parameter to detach plotting from verbosity
