# HSCOMBINE - Combine multispec files, removing cosmic rays in the process
# June 10, 2009
# By Doug Mink, SAO Telescope Data Center

procedure hscombine (images, output)

string	images=""	{prompt="Images to combine"}
file	output=""	{prompt="Output image"}
string	apertures = "1-300"	{prompt="Range of apertures to combine"}
string	combine = "average"	{prompt="Combine images (average|median|sum)"}
string	reject = "ccdclip"	{prompt="Reject method (none|minmax|ccdclip|crreject|sigclip|avsigclip|pclip)"}
string	scale = "none"	{prompt="Image scaling"}
string	sample = ""	{prompt="Wavelength sample regions for statistics"}
bool	saveold = yes	{prompt="Save uncombined files in Uncombined/ (y or n)"}
bool	deltemp = no	{prompt="Delete intermediate image files (y or n)"}
bool	verbose = yes	{prompt="Print process progress (y or n)"}
bool	debug = no	{prompt="Print information for debugging (y or n)"}

begin

string	imagelist, templist, history, keyword, args, dvalue
file	imout, imin, newfile, tempkey
bool	vb, db, samefile
int	nim, nfiles
real	dtime

	vb = verbose
	db = debug
	imagelist = images
	imout = output

	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

#  Make temporary list of input files and get name of first file
	templist = mktemp ("hscomblist")
	files (imagelist, >templist)
	list = templist
	i = fscan (list, imin)
	imgetp (imin, "APID1", verbose=db)

# Save APNUMi keywords with slit positions lost during scombine
	args = "-g " // imin // " @" // osfn ("hectospec$lib/apnumkeys")
	tempkey = mktemp ("hscombkey")
	gethead (args, >tempkey)

	nim = 1
	while (fscan (list,imin) != EOF) {
	    nim = nim + 1
	    }
	if (nim == 1 && imin == imout) {
	    samefile = yes
	    imout = mktemp ("hscombfile") // ".ms.fits:"
	    }
	else {
	    samefile = no
	    }

	if (vb) {
	    print ("HSCOMBINE: ", nim, " images combined into ",imout)
	    }

# Make sure that output file is multispec and quit if it already exists
	smatch (imout, ".ms.fits")
	if (!smatch.found) {
	    smatch (imout, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imout, 1, lroot)
		imout = substr (imout, 1, lroot) // ".ms.fits"
		}
	    else {
		imout = imout // ".ms.fits"
		}
	    }

        if (access (imout)) {
            print ("HSCOMBINE: Output file ", imin," already exists")
            return
            }


# Initialize scombine
	scombine.noutput = ""
	scombine.logfile = "STDOUT"
	scombine.apertures = apertures
	scombine.group = "apertures"
	scombine.combine = combine
	scombine.reject = reject
	scombine.first = yes
	scombine.w1 = INDEF
	scombine.w2 = INDEF
	scombine.dw = INDEF
	scombine.nw = INDEF
	scombine.log = no
	scombine.scale = scale
	scombine.zero = "none"
	scombine.weight = "none"
	scombine.sample = sample
	scombine.lthreshold = INDEF
	scombine.hthreshold = INDEF
	scombine.nlow = 1
	scombine.nhigh = 1
	scombine.nkeep = 1
	if (nim < 5) {
	    scombine.mclip = no
	    }
	else {
	    scombine.mclip = yes
	    }
	scombine.lsigma = 3.
	scombine.hsigma = 3.
	scombine.rdnoise = "2.7"
	scombine.gain = "1."
	scombine.snoise = "0."
	scombine.sigscale = 0.1
	scombine.pclip = -0.5
	scombine.grow = 0
	scombine.blank = 0.
	scombine.mode = "ql"

# Combine multispec images
	scombine (imagelist, imout)

# Copy APID1 from first input image to output image
	hedit (imout, "APID1", imgetp.value, add+, verify-)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " " // nim // " images scombined"
	hedit (imout, "HSCOMBIN", history, add+, verify-)

# Make directory to save input files, if needed
	if (saveold) {
	    if (!access ("Notcombined/")) {
		mkdir ("Notcombined")
		}
	    }

# Retrieve APNUMi keywords which were lost during scombine
	args = imout // " @" // tempkey
	sethead (args)

# Add list of files copied and copy HLIN and HPIXSHIF from each
	nfiles = 0
	dtime = 0.0
	list = templist
	while (fscan (list,imin) != EOF) {
	    nfiles = nfiles + 1
	    keyword = "HSCOMB" // nfiles
	    hedit (imout, keyword, imin, add+, verify-,show=debug)

	    keyword = "HSHIFT" // nfiles
	    imgetp (imin,"HPIXSHIF")
	    hedit (imout, keyword, imgetp.value, add+, verify-,show=debug)

	    keyword = "HLIN" // nfiles
	    imgetp (imin,"HLIN")
	    hedit (imout, keyword, imgetp.value, add+, verify-,show=debug)

	    keyword = "HSEXP" // nfiles
	    imgetp (imin,"EXPTIME")
	    hedit (imout, keyword, imgetp.value, add+, verify-,show=debug)

	    imgetp (imin,"DARKTIME")
	    dtime = dtime + real (imgetp.value)

	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imin = imin // ".fits"
		}
	    if (saveold) {
		newfile = "Notcombined/" // imin
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		movefiles (imin, "Notcombined")
		}
	    }

# Write summed dark time to header
	if (combine == "sum") {
	    dvalue = dtime
	    }
	else {
	    dvalue = dtime / real (nfiles)
	    }
	hedit (imout, "DARKTIME", dvalue, add+, verify-)

	if (samefile) {
	    if (access (imin)) {
		imdelete (imin)
		}
	    imrename (imout, imin)
	    }
	if (deltemp) {
	    delete (templist)
	    delete (tempkey)
	    }
end

# Jan 10 2005	New program to remove cosmic rays from spectra
# Feb 24 2005	Use Susan's defaults; add file transfer to subdirectory 
# Feb 24 2005	Add history, filename saving and APID1 transfer
# Feb 25 2005	Add scale and sample for images with different exposure levels
# Mar  7 2005	Allow single file to be input and output

# Dec 28 2006	Save APNUMi with slit positions and write into scombined file

# Mar 30 2007	Write HLIN and HPIXSHIF to HLINi and HSHIFTi of output header
# May 17 2007	Clean up code; allow file name extensions to be omitted
# May 22 2007	Check for package wcstools instead of task sethead
# May 24 2007	Declare undeclared variable args

# Jun 10 2009	Add HSEXPi keywords to save exposure times
# Jun 10 2009	Write mean or total dark time depending on value of combine
