# HSKYSHIFT - Compare image to reference and put shifts in header
# May 22, 2007
# By Doug Mink, SAO Telescope Data Center

procedure hskyshift (specdirs)

string	specdirs=""	{prompt="Directories with sky spectrum files for which to compute shifts"}

string	shiftline="O5577"	{prompt="Line to shift (O5577,O6300,OH7341,OH8399)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print shifts (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

struct *list1

begin

file	templist, tempdirs, tempfiles, tempemsao
file	imfile
string	imd, imds, history, tchar, nline, result
string	filetemp, arg, temp
real	shift, wl1
bool	vb, db, firstfile

# Load rvsao and wcstools if they haven't been loaded already
        if (!deftask ("emsao")) {
            rvsao (motd=no)
            }
        if (!defpac ("wcstools")) {
            wcstools (motd=no)
            }

	vb = verbose
	db = debug
	imds = specdirs
	if (shiftline == "O5577" || shiftline == "o5577") {
	    nline = " 9"
	    }
	else if (shiftline == "O6300" || shiftline == "o6300") {
	    nline = " 10"
	    }
	else if (shiftline == "OH7341" || shiftline == "oh7341") {
	    nline = " 11"
	    }
	else if (shiftline == "OH8399" || shiftline == "oh8399") {
	    nline = " 12"
	    }

	if (imds == "") {
	    print ("HSKYSHIFT: Find position of night sky line and shift spectra to match")
	    print ("Usage: hskyshift <input directories>")
	    print ("Usage: hskyshift @<input directory list file>")
	    return
	    }

#  Set up emsao parameters
	emsao.specnum = ""
	emsao.specband = 3
	emsao.skynum = ""
	emsao.skyband = 0
	emsao.specdir = ""
	emsao.linefit = yes
	emsao.fixbad = no
	emsao.badlines = "badlines.dat"
	emsao.renormalize = no
	emsao.st_lambda = INDEF
	emsao.end_lambda = INDEF
	emsao.nsmooth = 0
	emsao.vel_init = "guess"
	emsao.czguess = 0.
	emsao.cortemp = ""
	emsao.wspan = 10.
	emsao.linesig = 1.
	emsao.emsearch = "skysearch.dat"
	emsao.emlines = "hectosky.dat"
	emsao.linedir = "rvsao$lib"
	emsao.npfit = 2
	emsao.nlcont = 1
	emsao.esmooth = 2
	emsao.emcombine = ""
	emsao.mincont = 0.
	emsao.lwmin = 0.4
	emsao.lwmax = 1.7
	emsao.lsmin = 1.
	emsao.sigline = 0.
	emsao.disperr = 0.01
	emsao.vel_corr = "none"
	emsao.report_mode = 9
	emsao.archive = no
	emsao.save_vel = no
	emsao.verbose = vb
	emsao.device = "stdgraph"
	emsao.hardcopy = no
	emsao.displot = db
	emsao.plotter = "stdplot"
	emsao.dispmode = 2
	emsao.vel_plot = "emission"
	emsao.curmode = no
	emsao.dispem = yes
	emsao.dispabs = no
	emsao.ablines = "ablines.dat"
	emsao.obj_plot = no
	emsao.contsub_plot = no
	emsao.debug = no
	emsao.nsum = 1
	emsao.cursor = ""

#  Make temporary list of input directories
	tchar = substr (imds,1,1)
	if (tchar == '@') {
	    tempdirs = substr (imds,2,100)
	    }
	else {
            tempdirs = mktemp ("shiftlist")
            files (imds, >tempdirs)
	    }
        list = tempdirs
        while (fscan (list, imd) != EOF) {

#	Make list of files in this directory
            tempfiles = mktemp ("shiftfiles")
	    filetemp = imd // "/*.ms.fits"
	    files (filetemp, >tempfiles)
#	    print ("HSKYSHIFT: files -> ",tempfiles)

#	Compute shift of selected line using emsao
            tempemsao = mktemp ("shiftemsao")
	    if (db) {
		emsao.logfiles = "STDOUT," // tempemsao
		}
	    else {
		emsao.logfiles = tempemsao
		}
	    if (debug) {
		dpar emsao
		}
#	    print ("HSKYSHIFT: emsao -> ",tempemsao)
	    arg = "@" // tempfiles
	    emsao (arg)

# Compute median wavelength shift for selected emission line and read it back
#	    result = mktemp ("shift")
	    arg = "-ep " // tempemsao // nline
	    getcol (arg) | scan (shift)
#	    type (result)
#	    list1 = result
#	    fscan (list1, temp)
#	    shift = real (temp)

# Add shift to WCS in headers of files in this pointing directory
	    firstfile = yes
            list1 = tempfiles
            while (fscan (list1, imfile) != EOF) {
		if (firstfile) {
		    imgetp.default = "0"
		    imgetp (imfile, "CRVAL1")
		    wl1 = real (imgetp.value)
		    wl1 = wl1 - shift
		    firstfile = no
		    gettime ()
		    history = gettime.date // " " // gettime.time // " " // shift // " subtracted from wavelength"
		    if (vb) {
			print ("HSKYSHIFT: ", history)
			}
		    }
		imgetp.default = "0"
		imgetp (imfile, "SKYSHIFT")
		if (imgetp.value != "0") {
		    hedit (imfile,"HISTORY",imgetp.value,update=yes,verify=no,show=db)
		    if (vb) {
			print (imfile, ": ", imgetp.value)
			}
		    }
		hedit (imfile, "CRVAL1", wl1, update=yes, verify=no, show=db)
		hedit (imfile, "SKYSHIFT", history, add=yes, verify=no, show=db)
		}

# Delete temporary files for this pointing
	    if (deltemp) {
		delete (tempfiles)
		delete (tempemsao)
#		delete (result)
		}
	    }

	if (deltemp && tchar != '@') {
	    delete (tempdirs)
	    }
end

# May  6 2005	Program to compute night sky spectrum shifts and shift spectra
# Jun 22 2005	Change from mean to median of shifts
# Jun 30 2005	Load wcstools and rvsao in hectospec.cl
# Jul 21 2005	Drop unused parameter wlshift
# Jul 21 2005	Load rvsao and wcstools if they haven't been loaded already

# May 22 2007	Check for package wcstools instead of task sethead
